#######################################################
#        Internal functions                           #
#######################################################
statLogitMPL<-  function(theta,X,y,panel)
{   q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(X)), panel, sum) 
	out<-.C("stat_logit_MPL",
	  as.double(theta),
     as.integer(length(theta)),
     as.double(X),
     as.integer(nrow(X)),
     as.double(y),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     out=as.double(0.0),PACKAGE="panelMPL")$out
     return(out)
 }  	


dynLogitMPL<-  function(theta,Xe,y,y0,panel,S.i,theta.mle,alpha.mle)
{   q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(Xe)), panel, sum) 
	out<-.C("dyn_logit_MPL",
	  as.double(theta),
     as.integer(length(theta)),
     as.double(Xe),
     as.integer(nrow(Xe)),
     as.double(y),
     as.double(y0),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     as.double(S.i),
     as.integer(ncol(S.i)),
     as.double(theta.mle),
     as.double(alpha.mle),
     out=as.double(0.0),PACKAGE="panelMPL")$out
     return(out)
 }  

# use MC
dynLogitMPLS<-  function(theta,Xe,y,y0,panel,R,seed,theta.mle,alpha.mle)
{   
	set.seed(seed)
	q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(Xe)), panel, sum) 
	out<-.C("dyn_logit_MPL_simu",
	  as.double(theta),
     as.integer(length(theta)),
     as.double(Xe),
     as.integer(nrow(Xe)),
     as.double(y),
     as.double(y0),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     as.integer(R),
     as.double(theta.mle),
     as.double(alpha.mle),
     out=as.double(0.0),PACKAGE="panelMPL")$out
     return(out)
 }  

 
 
dynLogitPL<- function(theta,Xe,y,panel)
{ 
  alpha<-dynLogitAlpha(Xe,panel,y,theta,rep(0,length(unique(panel)))) 
  eta<- Xe %*% theta+alpha[panel]
  p<- p.range(plogis(eta))
  out<- sum(y*log(p)+(1-y)*log(1-p))
  return(-out)
  } 



dynLogitAlpha <- function(Xe,panel,y,theta,alpha.in)
{   
	 q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(Xe)), panel, sum) 
	 .C("dyn_logit_alpha",
	  as.double(alpha.in),
     as.double(theta),
     as.double(Xe),
     as.integer(length(theta)),
     as.double(y),
     as.integer(nrow(Xe)),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     as.double(10^-6),
     as.integer(100),
     alpha=as.double(rep(0.0,q)),PACKAGE="panelMPL")$alpha
   }


statProbitMPL<-  function(theta,X,y,panel,theta.mle,alpha.mle)
{   q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(X)), panel, sum) 
	out<-.C("stat_probit_MPL",
	  as.double(theta),
     as.integer(length(theta)),
     as.double(X),
     as.integer(nrow(X)),
     as.double(y),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     as.double(theta.mle),
     as.double(alpha.mle),
     out=as.double(0.0),PACKAGE="panelMPL")$out
     return(out)
 }  


dynProbitMPL<-  function(theta,Xe,y,y0,panel,S.i,theta.mle,alpha.mle)
{   q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(Xe)), panel, sum) 
	out<-.C("dyn_probit_MPL",
	  as.double(theta),
     as.integer(length(theta)),
     as.double(Xe),
     as.integer(nrow(Xe)),
     as.double(y),
     as.double(y0),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     as.double(S.i),
     as.integer(ncol(S.i)),
     as.double(theta.mle),
     as.double(alpha.mle),
     out=as.double(0.0),PACKAGE="panelMPL")$out
     return(out)
 }  
 
 
 
# use MC
dynProbitMPLS<-  function(theta,Xe,y,y0,panel,R,seed,theta.mle,alpha.mle)
{   
	set.seed(seed)
	q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(Xe)), panel, sum) 
	out<-.C("dyn_probit_MPL_simu",
	  as.double(theta),
     as.integer(length(theta)),
     as.double(Xe),
     as.integer(nrow(Xe)),
     as.double(y),
     as.double(y0),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     as.integer(R),
     as.double(theta.mle),
     as.double(alpha.mle),
     out=as.double(0.0),PACKAGE="panelMPL")$out
     return(out)
 }  


 
dynProbitPL<- function(theta,Xe,y,panel)
{ 
  alpha<-dynProbitAlpha(Xe,panel,y,theta,rep(0,length(unique(panel)))) 
  eta<- Xe %*% theta+alpha[panel]
  p<- p.range(pnorm(eta))
  out<- sum(y*log(p)+(1-y)*log(1-p))
  return(-out)
  } 





dynProbitAlpha <- function(Xe,panel,y,theta,alpha.in)
{   
	 q<- length(unique(panel)) 
	 ng<- tapply(rep(1,nrow(Xe)), panel, sum) 
	 .C("dyn_probit_alpha",
	  as.double(alpha.in),
     as.double(theta),
     as.double(Xe),
     as.integer(length(theta)),
     as.double(y),
     as.integer(nrow(Xe)),
     as.integer(ng),
     as.integer(q),
     as.integer(max(ng)),
     as.double(10^-6),
     as.integer(100),
     alpha=as.double(rep(0.0,q)),PACKAGE="panelMPL")$alpha
   }
