#######################################################
#        User-level functions                         #
#######################################################

panelMPL<- function(formula, data=list(), panel, model, method="nm", y0=NULL, start=NULL, R=NULL, seed=NULL)
{
	cl <- match.call()
	mf <- model.frame(formula=formula, data=data)
	x <- model.matrix( attr(mf, "terms"), data=mf)
	if(ncol(x)>1 & sd(x[,1])==0)  x <- as.matrix(model.matrix( attr(mf, "terms"), data=mf)[,-1])
	if(ncol(x)==1 & sd(x[,1])==0) x <- NULL
	if( (is.null(x)) & (is.element(model,c("statLogit","statProbit"))))
	stop("There are no structural parameters in the model")
	y <- model.response(mf)
	ord <-order(panel)
	panelo <- panel[ord] 
	panelu <- unique(panel)
	ordu <- order(panelu)
	n <- length(y)
    q<- length(unique(panelo))
    if(!is.null(x)) x <- as.matrix(x[ord,])
    y <- y[ord] 
    y0o <- y0[ordu]
    
    #drops the covariates with no within-panel variation
    flag <- NULL
    if(!is.null(x)) 
    {
    for(j in 1:ncol(x))
     { xj <- x[,j]
       x.m <- tapply(xj, panel, sd)
       if(sum(x.m)==0)
        flag <- c(flag,j)
     }
    } 
    if(!is.null(flag)) x <- as.matrix(x[,-flag])
    if( (is.null(x)) & (is.element(model,c("statLogit","statProbit"))))
	stop("There are no structural parameters in the model")
    
    if( (is.null(x)) & (is.element(model,c("truncReg"))))
	stop("There are no structural regression parameters in the model")
    
	list.mod<- c("dynLogit","dynProbit","statLogit","statProbit","truncReg","ar1")
    if(!is.element(model,list.mod)) stop("Unknown model", model)
    
    if (((model=="dynLogit")|(model=="dynProbit")) & (is.null(y0))) stop("Missing y0 values")
    
    #### obtains the lagged response 
    if((model=="dynLogit")|(model=="dynProbit"))
    {   
    	uni <- unique(panelo)
    	ym <- rep(NA,n)
        for(i in 1:q)
         {cond<- (panelo==uni[i])
          yi <- y[cond]
          mi <- length(yi)
          ymi <- rep(NA,mi)
          ymi[1] <- y0o[i]
          ymi[2:mi] <- yi[1:(mi-1)] 
          ym[cond] <- ymi
          }
        seeds <- seed  
		if(is.null( R )) S.i <- get.S(n/q)
		if(is.null(seeds)) seeds <- sample(1:10000,1) 
        xe<- cbind(x,ym)
        }
        
     if((model=="statLogit")|(model=="statProbit"))
    {   
        xe <- x
       }    
     
     if((model=="truncReg"))
    {    
       panelG <- get.g(panelo)
       xr <- x
       yr <- y
     }    
  
     if((model=="ar1"))
    {   
       panelG <- get.g(panelo)
       yr<- y
       xr <- if(!is.null(x)) x else NULL
     }   
        
    #### gets rid of uninformative strata
    if((model=="dynLogit")|(model=="dynProbit")|(model=="statLogit")|(model=="statProbit"))
    {    
       mean.y<- tapply(y,panelo,mean)
       cond<- !((mean.y==0)|(mean.y==1))
       ind<- match(panelo,names(cond)) 
       sele <- cond[ind]
       yr <- y[sele]
       xr <- as.matrix(xe[sele,])
       panelr <- panelo[sele]
       panelG <- get.g(panelr)   
       y0r <- y0o[cond]
       }
    
   
     ### final fitting via maxLik
     if(model=="dynLogit")  
    {  nr <- ncol(xr) 
       init <-  if(!is.null(start)) start else rep(0,nr)
       mle <- nlminb(init, dynLogitPL, Xe=xr, y=yr, panel=panelG)	
       mle$alpha <- dynLogitAlpha(xr, panelG, yr, mle$par, rep(0,length(panelG)) )
       if(is.null( R )) 
         {
         obj <- maxLik(dynLogitMPL,start=init,method=method,Xe=xr,y=yr,panel=panelG,y0=y0r,S.i=S.i,theta.mle=mle$par, alpha.mle=mle$alpha)
         obj$gradient <- grad(dynLogitMPL,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,S.i=S.i,theta.mle=mle$par, alpha.mle=mle$alpha)
        obj$hessian <- hessian(dynLogitMPL,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,S.i=S.i,theta.mle=mle$par, alpha.mle=mle$alpha)     
         }
       else
       {
         obj <- maxLik(dynLogitMPLS,start=init,method=method,Xe=xr,y=yr,panel=panelG,y0=y0r,R=R,seed=seeds, theta.mle=mle$par, alpha.mle=mle$alpha)
         obj$gradient <- grad(dynLogitMPLS,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,R=R,seed=seeds,theta.mle=mle$par, alpha.mle=mle$alpha)
        obj$hessian <- hessian(dynLogitMPLS,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,R=R,seed=seeds,theta.mle=mle$par, alpha.mle=mle$alpha)     
         }
       
       obj$mle <- mle$par  
       stri <- c() 
       if(nr>1) for(j in 1:(ncol(xr)-1)) stri<- c(stri, paste("x",j,sep="_"))
       names(obj$estimate) <- c(stri,"gamma")   
      }           
    
     if(model=="dynProbit")  
    {  nr <- if(is.null(xr)) 0 else ncol(xr) 
	   init <-  if(!is.null(start)) start else rep(0,nr)
       mle <- nlminb(init, dynProbitPL, Xe=xr, y=yr, panel=panelG)	
       mle$alpha <- dynProbitAlpha(xr, panelG, yr, mle$par, rep(0,length(panelG)) )
       if(is.null( R )) 
       {
       obj <- maxLik(dynProbitMPL,start=init,method=method,Xe=xr,y=yr,panel=panelG,y0=y0r,S.i=S.i,theta.mle=mle$par, alpha.mle=mle$alpha)
       obj$gradient <- grad(dynProbitMPL,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,S.i=S.i,theta.mle=mle$par, alpha.mle=mle$alpha)
       obj$hessian <- hessian(dynProbitMPL,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,S.i=S.i,theta.mle=mle$par, alpha.mle=mle$alpha)
       }
       else
       {
       obj <- maxLik(dynProbitMPLS,start=init,method=method,Xe=xr,y=yr,panel=panelG,y0=y0r,R=R,seed=seeds,theta.mle=mle$par, alpha.mle=mle$alpha)
       obj$gradient <- grad(dynProbitMPLS,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,R=R,seed=seeds,theta.mle=mle$par, alpha.mle=mle$alpha)
       obj$hessian <- hessian(dynProbitMPLS,obj$est,Xe=xr,y=yr,panel=panelG,y0=y0r,R=R,seed=seeds,theta.mle=mle$par, alpha.mle=mle$alpha)
       } 
       obj$mle <- mle$par   
       stri <- c() 
       if(nr>1) for(j in 1:(ncol(xr)-1)) stri<- c(stri, paste("x",j,sep="_"))
       names(obj$estimate) <- c(stri,"gamma") 
      }                   

     if(model=="statLogit")  
    {  
       init <-  if(!is.null(start)) start else rep(0,ncol(xr))
       mle <- nlminb(init, dynLogitPL, Xe=xr, y=yr, panel=panelG)	
       obj <- maxLik(statLogitMPL,start=init,method=method,X=xr,y=yr,panel=panelG)
       obj$gradient <- grad(statLogitMPL,obj$est,X=xr,y=yr,panel=panelG)
       obj$hessian <- hessian(statLogitMPL,obj$est,X=xr,y=yr,panel=panelG)
       obj$mle <- mle$par  
       stri <- c() 
       for(j in 1:ncol(xr)) stri<- c(stri, paste("x",j,sep="_"))
       names(obj$estimate) <- stri
        } 

     if(model=="statProbit")  
    {  init <-  if(!is.null(start)) start else rep(0,ncol(xr))
       mle <- nlminb(init, dynProbitPL, Xe=xr, y=yr, panel=panelG)	
       mle$alpha <- dynProbitAlpha(xr, panelG, yr, mle$par, rep(0,length(panelG)) )
       obj <- maxLik(statProbitMPL,start=init,method=method,X=xr,y=yr,panel=panelG,theta.mle=mle$par, alpha.mle=mle$alpha)
       obj$gradient <- grad(statProbitMPL,obj$est,X=xr,y=yr,panel=panelG,theta.mle=mle$par, alpha.mle=mle$alpha)
       obj$hessian <- hessian(statProbitMPL,obj$est,X=xr,y=yr,panel=panelG,theta.mle=mle$par, alpha.mle=mle$alpha)
       obj$mle <- mle$par   
       stri <- c() 
       for(j in 1:ncol(xr)) stri<- c(stri, paste("x",j,sep="_"))
       names(obj$estimate) <- stri
      } 
      
	if((model=="truncReg"))
	{  init <-  if(!is.null(start)) start else rep(0,ncol(xr)+1) 
	   obj <- maxLik(truncRegMPL,start=init,method=method,Xe=xr,y=yr,panel=panelG)
       obj$gradient <- grad(truncRegMPL,obj$est,Xe=xr,y=yr,panel=panelG)
       obj$hessian <- hessian(truncRegMPL,obj$est,Xe=xr,y=yr,panel=panelG) 
       mle <- nlminb(init,truncRegPL,Xe=xr,y=yr,panel=panelG)
       obj$mle <- mle$par  
       stri <- c() 
       for(j in 1:ncol(xr))
         stri<- c(stri, paste("x",j,sep="_"))
       names(obj$estimate) <- c(stri,"log(sigma)")
     } 

    if((model=="ar1"))
	{  nr <- if(is.null(xr)) 0 else ncol(xr) 
	   init <-  if(!is.null(start)) start else rep(0,nr+2)
	   lowi <- c(rep(-Inf,nr),-0.9999,-Inf)
	   upi <- c(rep(Inf,nr),0.9999,Inf)
	   obj <- nlminb(init,ar1MPL.nlminb,yr=yr,xr=xr,T=n/q,N=q,upper=upi,lower=lowi,control=list(rel.tol=10^-4))
	   obj <- maxLik(ar1MPL,start=obj$par,method=method,yr=yr,xr=xr,T=n/q,N=q)
       obj$gradient <- grad(ar1MPL,obj$est,xr=xr,yr=yr,T=n/q,N=q)
       obj$hessian <- hessian(ar1MPL,obj$est,xr=xr,yr=yr,T=n/q,N=q)
       mle <- nlminb(obj$est,ar1PL,yr=yr,xr=xr,T=n/q,N=q)
       obj$mle <- mle$par
       stri <- c() 
       if(nr>0) for(j in 1:ncol(xr)) stri<- c(stri, paste("x",j,sep="_"))
       names(obj$estimate) <- c(stri,"rho","log(sigma^2)")
       
     } 

    obj$call <- cl
    class(obj) <- c("panelMPL",class(obj))
    obj
}  

#### the following functions are  taken from the 'maxLik' package
print.summary.panelMPL <- function( x, ... ) {
	
   
   cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
   
   cat("--------------------------------------------\n")
   cat("Maximum Modified Profile Likelihood estimation\n") #this is the only change!
   cat(maximType(x), ", ", nIter(x), " iterations\n", sep="")
   cat("Return code ", returnCode(x), ": ", returnMessage(x), "\n", sep="")
   if(!is.null(x$estimate)) {
      cat("Modified Profile log-likelihood:", x$loglik, "\n")
      cat(x$NActivePar, " free parameters\n")
      cat("Estimates:\n")
      printCoefmat(x$estimate)
   }
   if(!is.null(x$constraints)) {
      cat("\nWarning: constrained likelihood estimation.",
          "Inference is probably wrong\n")
          
      cat("Constrained optimization based on", x$constraints$type,
          "\n")
      if(!is.null(x$constraints$code))
         cat("Return code:", x$constraints$code, "\n")
                           # note: this is missing for 'constrOptim'
      if(!is.null(x$constraints$message))
         cat(x$constraints$message, "\n")
                           # note: this is missing for 'constrOptim'
      cat(x$constraints$outer.iterations,
          " outer iterations, barrier value",
          x$constraints$barrier.value, "\n")
   }
   cat("--------------------------------------------\n")
}


summary.panelMPL <- function(object, eigentol=1e-12,... ) {
   ## object      object of class "panelMPL"
   ## 
   ## RESULTS:
   ## list of class "summary.panelMPL" with following components:
   ## maximum    : function value at optimum
   ## estimate   : estimated parameter values at optimum
   ## gradient   :           gradient at optimum
   ## code       : code of convergence
   ## message    : message, description of the code
   ## iterations : number of iterations
   ## type       : type of optimisation
   ##
   result <- object$maxim
   nParam <- length(coef <- coef(object))
   activePar <- activePar( object )
   if((object$code < 100) & !is.null(coef(object))) {
                           # in case of infinity at initial values, the coefs are not provided
       t <- coef(object)/stdEr(object)
       p <- 2*pnorm( -abs( t))
       t[!activePar(object)] <- NA
       p[!activePar(object)] <- NA
       results <- cbind("Estimate"=coef(object),
                        "Std. error"=stdEr(object),
                        "t value"=t, "Pr(> t)"=p)
   } else {
     results <- NULL
   }
   summary <- list(maximType=object$type,
                   call=object$call,
                   iterations=object$iterations,
                   returnCode=object$code,
                   returnMessage=object$message,
                   loglik=object$maximum,
                   estimate=results,
                   fixed=!activePar,
                   NActivePar=sum(activePar),
                   constraints=object$constraints)
   class(summary) <- "summary.panelMPL"
   summary
}


print.panelMPL <- function( x, ... ) {
	cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
   cat("Maximum Modified Profile Likelihood estimation\n")
   cat(maximType(x), ", ", nIter(x), " iterations\n", sep="")
   cat("Return code ", returnCode(x), ": ", returnMessage(x), "\n", sep="")
   if(!is.null(x$estimate)) {
      cat("Modified Profile log-likelihood:", x$maximum )
      cat( " (", sum( activePar( x ) ), " free parameter(s))\n", sep = "" )
      cat("Estimate(s):", x$estimate, "\n" )
   }
}

coef.summary.panelMPL <- function( object, ... ) {
   result <- object$estimate
   return( result )
}


simu.panelMPL <- function(X, panel, model, y0=NULL, beta, alpha, sigma=NULL, rho=NULL)
{
   alpha.v <- as.vector(alpha) 
   uni <- unique(panel)
   n <- nrow(X)
   q<- length(unique(panel))
   m <- n / q
   eta <- X %*% beta + alpha.v[panel]
   panelj <- rep(NA,n) #### 1:m within each panel
   for(i in 1:q)
   {
   	cond <- panel==uni[i]	
    panelj[cond] <- 1:m
   }
 
   
   if(model=="ar1")
    {  y<- rep(NA,n)
       for(i in 1:q)
        { cond <- panel==uni[i]	
    	  mu <- eta[cond]
    	  err <- arima.sim(list(ar=rho), sd=sigma, n=m)
    	  y[cond] <- mu + err
    	}
    }   
   if(model=="statLogit")
    {    
       y <- rbinom(n,size=1,prob=plogis(eta))
       }
   if(model=="statProbit")
    {    
       y <- rbinom(n,size=1,prob=pnorm(eta))
       }    
   if(model=="truncReg")
    {    
       y <- rtnorm(n,mean=eta,sd=sigma,lower=0)
    }    
  if(model=="dynLogit")
    {  y <- rep(NA,n)
       y0j<-y0
       for(j in 1:m)
         { 
   	      indj<- (1:n)[panelj==j]
   	      etaj<- alpha.v + as.matrix(X[indj,]) %*% beta + y0j * rho
          yj<-  rbinom(q,prob=plogis(etaj), size=1)
          y0j<-yj
          y[indj]<- yj
          }
      }
   if(model=="dynProbit")
    {  y <- rep(NA,n)
       y0j<-y0
       for(j in 1:m)
         { 
   	      indj<- (1:n)[panelj==j]
          etaj<- alpha.v + as.matrix(X[indj,]) %*% beta + y0j * rho
          yj<-  rbinom(q,prob=pnorm(etaj), size=1)
          y0j<-yj
          y[indj]<- yj
          }
    }      
    return(as.vector(y))  
}
