#include <R.h>
#include <Rmath.h>
#include <R_ext/Applic.h>
#include <R_ext/Lapack.h>

// auxiliar functions
#define MAT2(mat, i, j, nI) (mat[i+j*nI])

double eps=2.22e-15;

static double prange(double p)
{
  double mu;
  mu=p;
  
  if (mu<eps)
      mu=eps;
  if (mu>(1-eps))
      mu=(1-eps);
  return(mu);
 }   

void  panel_lag1( double *y,double y0, int dimData, double *yout)
{   double y0int;
    int i;
    
    y0int = y0;
    for(i=0; i< dimData; i++)
      {
        yout[i] = y0int;
	y0int = y[i];
           }
     }

// dynamic logit

static double lik_i_dlogit(double lambda,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, mu, l;
    int i,j;

    l=0.0;
    for(i=0; i<dimData; i++)
      {
       eta= lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       mu= plogis(eta,0,1,1,0);
       mu= prange(mu);		  
       l += y[i] * log(mu) + (1.0 - y[i]) * log(1-mu);
           }
    return(l); 
}


static double gr_i_dlogit( double lambda,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, mu, grad;
    int i,j;

  
    grad=0.0;
    
    for(i=0; i< dimData; i++)
      {
       eta=lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       mu= plogis(eta,0,1,1,0);
       mu= prange(mu);	
       grad += (y[i] - mu);
           }
    return(grad);
}


static double hess_i_dlogit( double lambda,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, mu, hess;
    int i,j;
    
    hess=0.0;
    
    for(i=0; i< dimData; i++)
      {
       eta=lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       mu= plogis(eta,0,1,1,0);
       mu= prange(mu);	
       hess += mu * (1.0 - mu);
           }
    return(hess);
}


static double FS_i_dlogit(double lambdain,
	  double *beta,       
	  int dimBeta,
	  double *X,
          int dimData,
          double *y,
	  double mytol,
	  int maxiter)
{  int i, iter;
  double lambdaold,lambdanew,s1,s2,f0,f1,lam,g0,g1,Delta,diff,H;


   lambdaold = lambdain;
   g0 = gr_i_dlogit(lambdaold,beta,dimBeta,X,dimData,y);
   f0 = R_pow_di(g0,2) * 0.5;
   iter = 0;
   
   while(iter< maxiter)
    { iter += 1;
      H = 1 / hess_i_dlogit(lambdaold,beta,dimBeta,X,dimData,y);
      g1 = gr_i_dlogit(lambdaold,beta,dimBeta,X,dimData,y);
      Delta = H * g1; 
      lambdanew= lambdaold + Delta;
      f1= R_pow_di(g1,2) * 0.5; 
      // backtracking loop 
      i=0;
      lam = -2.0 * f0;
      while(f1>(f0+0.0001*lam))
        {
	  //Rprintf("backtracking: %i, \n",i);
          i +=1;
          Delta = Delta * 0.5;
	  lambdanew= lambdaold + Delta;
	  lam = lam * 0.5;
          g1 = gr_i_dlogit(lambdanew,beta,dimBeta,X,dimData,y);
	  f1= R_pow_di(g1,2) * 0.5;
          if (i>100)
	    {Rprintf("backtracking failed:  \n");
	      break;}
         }
      diff = lambdaold - lambdanew;
      s1 = fabs(g1);
      if (s1< mytol)
	break;
      s1 = fabs(diff);
      s2 = fabs(lambdanew);
      if ( (s1/(1+s2))< mytol)
	break; 
      lambdaold = lambdanew;
      g0=g1;
      f0=f1;
     }
  return(lambdanew);
}




 void dyn_logit_alpha(double *ain,
       	         double *beta,
	         double *Xe,
	         int *dimBeta,
		 double *y,
		 int *dimData,
                 int *nG,
	         int *dimG,
		 int *maxnG,
		 double *mytol,
		 int *maxiter,
                 double *a)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xei=(double *) R_alloc(*maxnG * *dimBeta, sizeof(double));
  int i,j,ind,k,n,h;

  
  ind= 0;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<*dimBeta; h++)
	     MAT2(Xei,j,h,n) = MAT2(Xe,k,h,*dimData);
	 }
      
     
      a[i]= FS_i_dlogit(ain[i],beta,*dimBeta,Xei,n,yi,*mytol,*maxiter);
      ind += nG[i];
    }
  }
  

 static double lik_M_dyn_i_dlogit(double *theta,
	          int dimTheta,
		  double *X,
		  int dimData,
		  double *y,
		  double y0,
		  double *S,
		  int dimS,
		  double *thetasim,
		  double alphasim)
{ double l,alpha_s,info,P,score,scoresim,etasim,eta,mu,musim,I,l0;
  int i,j,k,ind;
  double *ys=(double *)  R_alloc(dimData, sizeof(double));
  double *Xe=(double *)  R_alloc(dimData * dimTheta, sizeof(double));
  double *ym=(double *)  R_alloc(dimData, sizeof(double));
  
    ind=dimTheta-1;
    panel_lag1(y,y0,dimData,ym);
    for(i=0; i<dimData; i++)
       { for (j=0; j< ind ; j++)
	   MAT2(Xe,i,j,dimData) = MAT2(X,i,j,dimData);
	 MAT2(Xe,i,ind,dimData) = ym[i];}
    alpha_s= FS_i_dlogit(0.0,theta,dimTheta,Xe,dimData,y,0.000001,100); 
    l = lik_i_dlogit(alpha_s,theta,dimTheta,Xe,dimData,y);
    info =  hess_i_dlogit(alpha_s,theta,dimTheta,Xe,dimData,y);
    I = 0.0;
    for(j=0; j<dimS; j++)
      {
	for(i=0; i<dimData; i++)
	 ys[i] = MAT2(S,i,j,dimData);
        panel_lag1(ys,y0,dimData,ym); 
	for(i=0; i<dimData; i++)
          MAT2(Xe,i,ind,dimData) = ym[i];
        l0 = lik_i_dlogit(alphasim,thetasim,dimTheta,Xe,dimData,ys);
        P = exp(l0);
        score=0.0;
        scoresim=0.0;
        for(i=0; i<dimData; i++)
	  {  eta= alpha_s;
             etasim = alphasim; 
             for(k=0; k< dimTheta; k++)
                { eta += MAT2(Xe,i,k,dimData) * theta[k];
                  etasim += MAT2(Xe,i,k,dimData) * thetasim[k];
                }
             mu= plogis(eta,0,1,1,0);
             mu= prange(mu);	
             musim= plogis(etasim,0,1,1,0);
             musim= prange(musim);	
             score += ys[i]-mu;
             scoresim +=  ys[i]-musim;
	  }
        I += P * score * scoresim;
       }
    l += 0.5 * log(info) - log(I);
    return(l);   
}



void dyn_logit_MPL(double *theta,
       	               int *dimTheta,
                       double *X,
		    int *dimData,
		       double *y,
		       double *y0,
		       int *nG,
		       int *dimG,
		       int *maxnG,
		       double *S,
		       int *dimS,
		       double *thetaMle,
		       double *alphaMle,
                       double *l)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xi=(double *) R_alloc(*maxnG * *dimTheta, sizeof(double));
  int i,j,ind,k,n,h,dm1;
  double y0i,alphai;

  *l=0.0;
  ind= 0;
  dm1 = *dimTheta-1;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      y0i = y0[i];
      alphai = alphaMle[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<dm1; h++)
	     MAT2(Xi,j,h,n) = MAT2(X,k,h,*dimData);
	 }
      *l += lik_M_dyn_i_dlogit(theta,*dimTheta,Xi,n,yi,y0i,S,*dimS,thetaMle,alphai);
      ind += nG[i];
    }
  }
  
 /* An  attempt to obtain   M via MC simulation */
 static double lik_M_dyn_i_dlogit_simu(double *theta,
	                                   int dimTheta,
		                               double *X,
		                               int dimData,
		                               double *y,
		                               double y0,
		                               int R,
		                               double *thetasim,
		                               double alphasim)
{ double l,alpha_s,info,score,scoresim,etasim,eta,mu,musim,I,yms;
  int i,j,k,ind;
  double *ys=(double *)  R_alloc(dimData, sizeof(double));
  double *Xe=(double *)  R_alloc(dimData * dimTheta, sizeof(double));
  double *ym=(double *)  R_alloc(dimData, sizeof(double));
  double *etaF=(double *)  R_alloc(dimData, sizeof(double));
  double *etasimF=(double *)  R_alloc(dimData, sizeof(double));
   
    GetRNGstate();  
    ind=dimTheta-1;
    panel_lag1(y,y0,dimData,ym);
    for(i=0; i<dimData; i++)
      {
       for(j=0; j< ind ; j++)
	     MAT2(Xe,i,j,dimData) = MAT2(X,i,j,dimData);
	    MAT2(Xe,i,ind,dimData) = ym[i];
	   }
    alpha_s= FS_i_dlogit(0.0,theta,dimTheta,Xe,dimData,y,0.000001,100); 
    l = lik_i_dlogit(alpha_s,theta,dimTheta,Xe,dimData,y);
    info =  hess_i_dlogit(alpha_s,theta,dimTheta,Xe,dimData,y);
    l += 0.5 * log(info);
    /* now the I factor   */
    for(i=0; i<dimData; i++)
	  { 
	   eta= alpha_s;
       etasim = alphasim; 
       for(k=0; k<ind; k++)
          { 
           eta += MAT2(X,i,k,dimData) * theta[k];
           etasim += MAT2(X,i,k,dimData) * thetasim[k];
                }
       etaF[i] = eta;
       etasimF[i] = etasim;
	  }
    I = 0.0;
    for(j=0; j<R; j++)
      {
      	score=0.0;
        scoresim=0.0;
        for(i=0; i<dimData; i++)
	      {               
	      	if(i==0)
               yms = y0;
             else 
               yms = ys[i-1];
             eta = etaF[i] + theta[ind] * yms;
             etasim = etasimF[i]+thetasim[ind] * yms; 
             mu= plogis(eta,0,1,1,0);
             mu= prange(mu);	
             musim= plogis(etasim,0,1,1,0);
             musim= prange(musim);	
             ys[i] = rbinom(1.0,musim);
             score += ys[i]-mu;
             scoresim +=  ys[i]-musim;
	       }
        I += score * scoresim / R;
       }
    if(I>0) 
      l +=  - log( I );
           
    PutRNGstate();
    return(l);   
}
  
  

void dyn_logit_MPL_simu(double *theta,
       	               int *dimTheta,
                       double *X,
		    		   int *dimData,
		       		   double *y,
		       		   double *y0,
		       		   int *nG,
		       		   int *dimG,
		       		   int *maxnG,
		       		   int *R,
		       		   double *thetaMle,
		               double *alphaMle,
                       double *l)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xi=(double *) R_alloc(*maxnG * *dimTheta, sizeof(double));
  int i,j,ind,k,n,h,dm1;
  double y0i,alphai;

  *l=0.0;
  ind= 0;
  dm1 = *dimTheta-1;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      y0i = y0[i];
      alphai = alphaMle[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<dm1; h++)
	     MAT2(Xi,j,h,n) = MAT2(X,k,h,*dimData);
	 }
      *l += lik_M_dyn_i_dlogit_simu(theta,*dimTheta,Xi,n,yi,y0i,*R,thetaMle,alphai);
      ind += nG[i];
    }
   
   }  
  
  
// dynamic probit

static double lik_i_dprobit(double lambda,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, mu, l;
    int i,j;

    l=0.0;
    for(i=0; i<dimData; i++)
      {
       eta= lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       mu= pnorm(eta,0,1,1,0);
       mu= prange(mu);		  
       l += y[i] * log(mu) + (1.0 - y[i]) * log(1-mu);
           }
    return(l); 
}


static double gr_i_dprobit( double lambda,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, mu, dmu, grad;
    int i,j;

  
    grad=0.0;
    
    for(i=0; i< dimData; i++)
      {
       eta=lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       mu= pnorm(eta,0,1,1,0);
       mu= prange(mu);	
       dmu = dnorm(eta,0,1,0);
       grad += dmu * (y[i] - mu) / (mu * (1-mu) );
           }
    return(grad);
}


static double hess_i_dprobit( double lambda,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, mu, dmu, num, A, B, hess;
    int i,j;
    
    hess=0.0;
    
    for(i=0; i< dimData; i++)
      {
       eta=lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       mu= pnorm(eta,0,1,1,0);
       mu= prange(mu);	
       dmu = dnorm(eta,0,1,0);
       A = (-1) * dmu * (dmu + eta * (y[i] - mu));
       B = dmu * (1 - 2 * mu);
       num = A * (mu - mu * mu) - dmu * (y[i] - mu) * B;
       hess += num / (mu * (1-mu) * mu * (1-mu) ) ;
           }
    return(-1.0 * hess);
}


static double FS_i_dprobit(double lambdain,
	  double *beta,       
	  int dimBeta,
	  double *X,
          int dimData,
          double *y,
	  double mytol,
	  int maxiter)
{  int i, iter;
  double lambdaold,lambdanew,s1,s2,f0,f1,lam,g0,g1,Delta,diff,H;


   lambdaold = lambdain;
   g0 = gr_i_dprobit(lambdaold,beta,dimBeta,X,dimData,y);
   f0 = R_pow_di(g0,2) * 0.5;
   iter = 0;
   
   while(iter< maxiter)
    { iter += 1;
      H = 1 / hess_i_dprobit(lambdaold,beta,dimBeta,X,dimData,y);
      g1 = gr_i_dprobit(lambdaold,beta,dimBeta,X,dimData,y);
      Delta = H * g1; 
      lambdanew= lambdaold + Delta;
      f1= R_pow_di(g1,2) * 0.5; 
      // backtracking loop 
      i=0;
      lam = -2.0 * f0;
      while(f1>(f0+0.0001*lam))
        {
	  //Rprintf("backtracking: %i, \n",i);
          i +=1;
          Delta = Delta * 0.5;
	      lambdanew= lambdaold + Delta;
	      lam = lam * 0.5;
          g1 = gr_i_dprobit(lambdanew,beta,dimBeta,X,dimData,y);
	      f1= R_pow_di(g1,2) * 0.5;
          if (i>100)
	    {Rprintf("backtracking failed:  \n");
	      break;}
         }
      diff = lambdaold - lambdanew;
      s1 = fabs(g1);
      if (s1< mytol)
	break;
      s1 = fabs(diff);
      s2 = fabs(lambdanew);
      if ( (s1/(1+s2))< mytol)
	break; 
      lambdaold = lambdanew;
      g0=g1;
      f0=f1;
     }
   return(lambdanew);
}



 void dyn_probit_alpha(double *ain,
       	         double *beta,
	         double *Xe,
	         int *dimBeta,
		 double *y,
		 int *dimData,
                 int *nG,
	         int *dimG,
		 int *maxnG,
		 double *mytol,
		 int *maxiter,
                 double *a)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xei=(double *) R_alloc(*maxnG * *dimBeta, sizeof(double));
  int i,j,ind,k,n,h;

  
  ind= 0;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<*dimBeta; h++)
	     MAT2(Xei,j,h,n) = MAT2(Xe,k,h,*dimData);
	 };
      a[i]= FS_i_dprobit(ain[i],beta,*dimBeta,Xei,n,yi,*mytol,*maxiter);
      ind += nG[i];
    };
  }



 static double lik_M_dyn_i_dprobit(double *theta,
	          int dimTheta,
		  double *X,
		  int dimData,
		  double *y,
		  double y0,
		  double *S,
		  int dimS,
		  double *thetasim,
		  double alphasim)
{ double l,alpha_s,info,P,score,scoresim,etasim,eta,mu,musim,dmu,dmusim,I,l0;
  int i,j,k,ind;
  double *ys=(double *)  R_alloc(dimData, sizeof(double));
  double *Xe=(double *)  R_alloc(dimData * dimTheta, sizeof(double));
  double *ym=(double *)  R_alloc(dimData, sizeof(double));
  
    ind=dimTheta-1;
    panel_lag1(y,y0,dimData,ym);
    for(i=0; i<dimData; i++)
       { for (j=0; j< ind ; j++)
	   MAT2(Xe,i,j,dimData) = MAT2(X,i,j,dimData);
	 MAT2(Xe,i,ind,dimData) = ym[i];}
    alpha_s= FS_i_dprobit(0.0,theta,dimTheta,Xe,dimData,y,0.000001,100); 
    l = lik_i_dprobit(alpha_s,theta,dimTheta,Xe,dimData,y);
    info =  hess_i_dprobit(alpha_s,theta,dimTheta,Xe,dimData,y);
    l += 0.5 * log(info);
    I = 0.0;
    for(j=0; j<dimS; j++)
      {
	for(i=0; i<dimData; i++)
	 ys[i] = MAT2(S,i,j,dimData);
        panel_lag1(ys,y0,dimData,ym); 
	for(i=0; i<dimData; i++)
          MAT2(Xe,i,ind,dimData) = ym[i];
        l0 = lik_i_dprobit(alphasim,thetasim,dimTheta,Xe,dimData,ys);
        P = exp(l0);
        score=0.0;
        scoresim=0.0;
        for(i=0; i<dimData; i++)
	  {  eta= alpha_s;
             etasim = alphasim; 
             for(k=0; k< dimTheta; k++)
                { eta += MAT2(Xe,i,k,dimData) * theta[k];
                  etasim += MAT2(Xe,i,k,dimData) * thetasim[k];
                }
             mu= pnorm(eta,0,1,1,0);
             dmu = dnorm(eta,0,1,0);
             mu= prange(mu);	
             musim= pnorm(etasim,0,1,1,0);
             dmusim = dnorm(etasim,0,1,0);
             musim= prange(musim);	
             score += (ys[i]-mu) * dmu / (mu * (1-mu));
             scoresim +=  (ys[i]-musim) * dmusim / (musim * (1-musim));
	  }
        I += P * score * scoresim;
       }
    l += - log(I);
    return(l);   
}


void dyn_probit_MPL(double *theta,
       	               int *dimTheta,
                       double *X,
		    int *dimData,
		       double *y,
		       double *y0,
		       int *nG,
		       int *dimG,
		       int *maxnG,
		       double *S,
		       int *dimS,
		       double *thetaMle,
		       double *alphaMle,
                       double *l)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xi=(double *) R_alloc(*maxnG * *dimTheta, sizeof(double));
  int i,j,ind,k,n,h,dm1;
  double y0i,alphai;

  *l=0.0;
  ind= 0;
  dm1 = *dimTheta-1;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      y0i = y0[i];
      alphai = alphaMle[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<dm1; h++)
	     MAT2(Xi,j,h,n) = MAT2(X,k,h,*dimData);
	 }
      *l += lik_M_dyn_i_dprobit(theta,*dimTheta,Xi,n,yi,y0i,S,*dimS,thetaMle,alphai);
      ind += nG[i];
    }
  }
  
 

 /* An  attempt to obtain   M via MC simulation */
 static double lik_M_dyn_i_dprobit_simu(double *theta,
	                                   int dimTheta,
		                               double *X,
		                               int dimData,
		                               double *y,
		                               double y0,
		                               int R,
		                               double *thetasim,
		                               double alphasim)
{ double l,alpha_s,info,score,scoresim,etasim,eta,mu,musim,I,yms,dmu,dmusim;
  int i,j,k,ind;
  double *ys=(double *)  R_alloc(dimData, sizeof(double));
  double *Xe=(double *)  R_alloc(dimData * dimTheta, sizeof(double));
  double *ym=(double *)  R_alloc(dimData, sizeof(double));
  double *etaF=(double *)  R_alloc(dimData, sizeof(double));
  double *etasimF=(double *)  R_alloc(dimData, sizeof(double));
  
    GetRNGstate();  
    ind=dimTheta-1;
    panel_lag1(y,y0,dimData,ym);
    for(i=0; i<dimData; i++)
      {
       for(j=0; j< ind ; j++)
	     MAT2(Xe,i,j,dimData) = MAT2(X,i,j,dimData);
	    MAT2(Xe,i,ind,dimData) = ym[i];
	   }
    alpha_s= FS_i_dprobit(0.0,theta,dimTheta,Xe,dimData,y,0.000001,100); 
    l = lik_i_dprobit(alpha_s,theta,dimTheta,Xe,dimData,y);
    info =  hess_i_dprobit(alpha_s,theta,dimTheta,Xe,dimData,y);
    l += 0.5 * log(info);
    /* now the I factor   */
    for(i=0; i<dimData; i++)
	  { 
	   eta= alpha_s;
       etasim = alphasim; 
       for(k=0; k<ind; k++)
          { 
           eta += MAT2(X,i,k,dimData) * theta[k];
           etasim += MAT2(X,i,k,dimData) * thetasim[k];
                }
       etaF[i] = eta;
       etasimF[i] = etasim;
	  }
    I = 0.0;
    for(j=0; j<R; j++)
      {
      	score=0.0;
        scoresim=0.0;
        for(i=0; i<dimData; i++)
	      {               
	      	if(i==0)
               yms = y0;
             else 
               yms = ys[i-1];
             eta = etaF[i] + theta[ind] * yms;
             etasim = etasimF[i]+thetasim[ind] * yms; 
             mu= pnorm(eta,0,1,1,0);
             mu= prange(mu);	
             dmu = dnorm(eta,0,1,0);
             musim= pnorm(etasim,0,1,1,0);
             musim= prange(musim);	
             dmusim = dnorm(etasim,0,1,0);
             ys[i] = rbinom(1.0,musim);
             score += (ys[i]-mu) * dmu / (mu * (1-mu));
             scoresim +=  (ys[i]-musim) * dmusim / (musim * (1-musim));
	       }
        I += score * scoresim / R;
       }
    if(I>0)   
       l += - log( I );
        
    PutRNGstate();
    return(l);   
 }
 
 
 void dyn_probit_MPL_simu(double *theta,
       	               int *dimTheta,
                       double *X,
		    		   int *dimData,
		       		   double *y,
		       		   double *y0,
		       		   int *nG,
		       		   int *dimG,
		       		   int *maxnG,
		       		   int *R,
		       		   double *thetaMle,
		               double *alphaMle,
                       double *l)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xi=(double *) R_alloc(*maxnG * *dimTheta, sizeof(double));
  int i,j,ind,k,n,h,dm1;
  double y0i,alphai;

  *l=0.0;
  ind= 0;
  dm1 = *dimTheta-1;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      y0i = y0[i];
      alphai = alphaMle[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<dm1; h++)
	     MAT2(Xi,j,h,n) = MAT2(X,k,h,*dimData);
	 }
      *l += lik_M_dyn_i_dprobit_simu(theta,*dimTheta,Xi,n,yi,y0i,*R,thetaMle,alphai);
      ind += nG[i];
  //    Rprintf("%i,%f,\n",i,*l);
    }
   
   }  
    
  
//static logit
    
static double lik_M_stat_logit_i(double *theta,
	          int dimTheta,
		  double *X,
		  int dimData,
		  double *y)
{ double alpha_s,info,I,l;

    alpha_s= FS_i_dlogit(0.0,theta,dimTheta,X,dimData,y,0.000001,100); 
    l = lik_i_dlogit(alpha_s,theta,dimTheta,X,dimData,y);
    info = hess_i_dlogit(alpha_s,theta,dimTheta,X,dimData,y);
    I = 1;
    l += 0.5 * log(info) - log(I);
    return(l); 
   }



void stat_logit_MPL(double *theta,
       	               int *dimTheta,
                       double *X,
		       int *dimData,
		       double *y,
		       int *nG,
		       int *dimG,
		       int *maxnG,
		              double *l)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xi=(double *) R_alloc(*maxnG * *dimTheta, sizeof(double));
  int i,j,ind,k,n,h;
  

  *l=0.0;
  ind= 0;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	       yi[j] = y[k];
           for(h=0; h<*dimTheta; h++)
	         MAT2(Xi,j,h,n) = MAT2(X,k,h,*dimData);
	       }
      *l += lik_M_stat_logit_i(theta,*dimTheta,Xi,n,yi);
      ind += nG[i];
    }
  }

//static probit

static double I_i( double lambda,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double lambdamle,
	    double *betamle)
{   double eta, mu, fu, I, den, muh, etah, fuh;
    int i,j;
    
    I=0.0;
    for(i=0; i< dimData; i++)
      {
       eta=lambda; 
       etah=lambdamle;
       for(j=0; j< dimBeta; j++)
         {eta += MAT2(X,i,j,dimData) * beta[j];
          etah += MAT2(X,i,j,dimData) * betamle[j];
          }
       mu= pnorm(eta,0,1,1,0);
       mu= prange(mu);
       muh=pnorm(etah,0,1,1,0);
       muh= prange(muh);
       fu= dnorm(eta,0,1,0);
       fuh= dnorm(etah,0,1,0);
       den= mu * (1.0 - mu);
       I += fu * fuh / den ;
           }
    return(I);
}



    
static double lik_M_stat_probit_i(double *theta,
	          int dimTheta,
		  double *X,
		  int dimData,
		  double *y,
		  double alphamle,
		  double *betamle)
{ double alpha_s,info,I,l;

    alpha_s= FS_i_dprobit(0.0,theta,dimTheta,X,dimData,y,0.000001,100); 
    l = lik_i_dprobit(alpha_s,theta,dimTheta,X,dimData,y);
    info = hess_i_dprobit(alpha_s,theta,dimTheta,X,dimData,y);
    I = I_i(alpha_s,theta,dimTheta,X,dimData,alphamle,betamle);
    l += 0.5 * log(info) - log(I);
    return(l); 
   }


void stat_probit_MPL(double *theta,
       	               int *dimTheta,
                       double *X,
		               int *dimData,
		               double *y,
		               int *nG,
		               int *dimG,
		               int *maxnG,
		               double *thetamle,
		               double *alphamle,
		              double *l)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xi=(double *) R_alloc(*maxnG * *dimTheta, sizeof(double));
  int i,j,ind,k,n,h;
  double alphai;

  *l=0.0;
  ind= 0;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      alphai = alphamle[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	       yi[j] = y[k];
           for(h=0; h<*dimTheta; h++)
	         MAT2(Xi,j,h,n) = MAT2(X,k,h,*dimData);
	       }
	  *l += lik_M_stat_probit_i(theta,*dimTheta,Xi,n,yi,alphai,thetamle);
      ind += nG[i];
    }
  }
  
  

//trunc Regression

static double lik_i_treg(double lambda, 
		double *sigma,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, etas, lmu, l;
    int i,j;

    l=0.0;
    for(i=0; i<dimData; i++)
      {
       eta= lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       etas= eta/sigma[0];
       lmu= pnorm(etas,0,1,1,1);
       l += -log(sigma[0]) -0.5*R_pow_di((y[i]-eta)/sigma[0],2)-lmu;
           }
    return(l); 
}



static double gr_i_treg( double lambda,
		double *sigma,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, etas, mu, grad;
    int i,j;

  
    grad=0.0;
    
    for(i=0; i< dimData; i++)
      {
       eta=lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       etas= eta/sigma[0];
       mu= pnorm(etas,0,1,1,0);
       mu= prange(mu);	
       grad += (y[i] - eta)/R_pow_di(sigma[0],2)-dnorm(etas,0,1,0)/sigma[0]/mu;
           }
    return(grad);
}


static double hess_i_treg( double lambda,
		double *sigma,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, etas, mu, dmu, hess;
    int i,j;
    
    hess=0.0;
    
    for(i=0; i< dimData; i++)
      {
       eta=lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       etas= eta/sigma[0];
       mu= pnorm(etas,0,1,1,0);
       mu= prange(mu);	
       dmu=dnorm(etas,0,1,0);
       hess += (1.0 - dmu/R_pow_di(mu,2)*(etas*mu+dmu))/R_pow_di(sigma[0],2);
           }
    return(hess);
}


 
static double FS_i_treg(double lambdain,
	  double *sigma,
	  double *beta,       
	  int dimBeta,
	  double *X,
          int dimData,
          double *y,
	  double mytol,
	  int maxiter)
{  int i, iter;
  double lambdaold,lambdanew,s1,s2,f0,f1,lam,g0,g1,Delta,diff,H;


   lambdaold = lambdain;
   g0 = gr_i_treg(lambdaold,sigma,beta,dimBeta,X,dimData,y);
   f0 = R_pow_di(g0,2) * 0.5;
   iter = 0;
   
   while(iter< maxiter)
    { iter += 1;
      H = 1 / hess_i_treg(lambdaold,sigma,beta,dimBeta,X,dimData,y);
      g1 = gr_i_treg(lambdaold,sigma,beta,dimBeta,X,dimData,y);
      Delta = H * g1; 
      lambdanew= lambdaold + Delta;
      f1= R_pow_di(g1,2) * 0.5; 
      // backtracking loop 
      i=0;
      lam = -2.0 * f0;
      while(f1>(f0+0.0001*lam))
        {
	  //Rprintf("backtracking: %i, \n",i);
          i +=1;
          Delta = Delta * 0.5;
	  lambdanew= lambdaold + Delta;
	  lam = lam * 0.5;
          g1 = gr_i_treg(lambdanew,sigma,beta,dimBeta,X,dimData,y);
	  f1= R_pow_di(g1,2) * 0.5;
          if (i>100)
	    {Rprintf("backtracking failed:  \n");
	      break;}
         }
      diff = lambdaold - lambdanew;
      s1 = fabs(g1);
      if (s1< mytol)
	break;
      s1 = fabs(diff);
      s2 = fabs(lambdanew);
      if ( (s1/(1+s2))< mytol)
	break; 
      lambdaold = lambdanew;
      g0=g1;
      f0=f1;
     }
 
   return(lambdanew);
}


void alpha_null_treg(double *ain,
				 double *sigma,
       	         double *beta,
	         double *Xe,
	         int *dimBeta,
		 double *y,
		 int *dimData,
                 int *nG,
	         int *dimG,
		 int *maxnG,
		 double *mytol,
		 int *maxiter,
                 double *a)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xei=(double *) R_alloc(*maxnG * *dimBeta, sizeof(double));
  int i,j,ind,k,n,h;

  
  ind= 0;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<*dimBeta; h++)
	     MAT2(Xei,j,h,n) = MAT2(Xe,k,h,*dimData);
	 }
      a[i]= FS_i_treg(ain[i],sigma,beta,*dimBeta,Xei,n,yi,*mytol,*maxiter);
      ind += nG[i];
    }
  }
  


void trunc_reg_PL(double *ain,
				 double *sigma,
       	         double *beta,
	         double *Xe,
	         int *dimBeta,
		 double *y,
		 int *dimData,
                 int *nG,
	         int *dimG,
		 int *maxnG,
		 double *mytol,
		 int *maxiter,
                 double *lP)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xei=(double *) R_alloc(*maxnG * *dimBeta, sizeof(double));
  double a;
  int i,j,ind,k,n,h;
  
  a=0;
  ind= 0;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<*dimBeta; h++)
	     MAT2(Xei,j,h,n) = MAT2(Xe,k,h,*dimData);
	 }
      
     
      a= FS_i_treg(ain[i],sigma,beta,*dimBeta,Xei,n,yi,*mytol,*maxiter);
      lP[0] -= lik_i_treg(a,sigma,beta,*dimBeta,Xei,n,yi);
      ind += nG[i];
    }
    
  }
  



static double likM_i_treg(double lambda, 
		double *sigma,
	    double *beta,
	    int dimBeta,
	    double *X,
	    int dimData,
	    double *y)
{   double eta, etas, lmu, lden, l, mod;
    int i,j;

    mod=0.0;
    l=0.0;
    for(i=0; i<dimData; i++)
      {
       eta= lambda; 
       for(j=0; j< dimBeta; j++)
         eta += MAT2(X,i,j,dimData) * beta[j];
       etas = eta/sigma[0];
       lmu = pnorm(etas,0,1,1,1);
       lden = dnorm(etas,0,1,1);
       mod += 1- R_pow_di(exp(lden-lmu),2)-exp(lden-lmu)*etas;
       l += -log(sigma[0]) -0.5*R_pow_di((y[i]-eta)/sigma[0],2)-lmu;
           }
    return(l+log(sigma[0])+0.5*log(mod)); 
}



void trunc_reg_MPL(double *ain,
				 double *sigma,
       	         double *beta,
	         double *Xe,
	         int *dimBeta,
		 double *y,
		 int *dimData,
                 int *nG,
	         int *dimG,
		 int *maxnG,
		 double *mytol,
		 int *maxiter,
                 double *lM)
{ 
 
  double *yi=(double *) R_alloc(*maxnG, sizeof(double));
  double *Xei=(double *) R_alloc(*maxnG * *dimBeta, sizeof(double));
  double a;
  int i,j,ind,k,n,h;
  
  a=0;
  ind= 0;
  for(i=0; i< *dimG; i++)
    { n = nG[i];
      for(j=0; j<n; j++)
         { k = j + ind;
	   yi[j] = y[k];
           for(h=0; h<*dimBeta; h++)
	     MAT2(Xei,j,h,n) = MAT2(Xe,k,h,*dimData);
	 }
     
      a= FS_i_treg(ain[i],sigma,beta,*dimBeta,Xei,n,yi,*mytol,*maxiter);
      
      lM[0] -= likM_i_treg(a,sigma,beta,*dimBeta,Xei,n,yi);
      ind += nG[i];
    }
    
  }



